Package: emdbook
Type: Package
Title: Support Functions and Data for "Ecological Models and Data"
Version: 1.3.14
LazyData: yes
Authors@R: c(person("Ben","Bolker",email="bolker@mcmaster.ca", role=c("aut","cre")),
	   person("Sang Woo","Park",role="ctb"),
	   person("James","Vonesh",role="dtc"),
	   person("Jacqueline","Wilson",role="dtc"),
	   person("Russ","Schmitt",role="dtc"),
	   person("Sally","Holbrook",role="dtc"),
	   person("James D.","Thomson",role="dtc"),
	   person("R. Scot","Duncan",role="dtc")
	   )
Description: Auxiliary functions and data sets for "Ecological Models and Data", a book presenting maximum likelihood estimation and related topics for ecologists (ISBN 978-0-691-12522-0).
Suggests: R2jags, ellipse, SuppDists, numDeriv, testthat, rgl
Imports: MASS, lattice, plyr, coda, bbmle
License: GPL
URL: https://math.mcmaster.ca/bolker/emdbook
NeedsCompilation: no
Packaged: 2025-07-23 13:42:20 UTC; bolker
Author: Ben Bolker [aut, cre],
  Sang Woo Park [ctb],
  James Vonesh [dtc],
  Jacqueline Wilson [dtc],
  Russ Schmitt [dtc],
  Sally Holbrook [dtc],
  James D. Thomson [dtc],
  R. Scot Duncan [dtc]
Maintainer: Ben Bolker <bolker@mcmaster.ca>
Repository: CRAN
Date/Publication: 2025-07-23 14:50:02 UTC

## exportPattern("^[^\\.]")
export(lambertW,lWasymp,lambertW_base,
       get.emdbook.packages,
       deltavar,deltamethod,
       metropSB,trcoef,
       lseq,scinot,axis.scinot,
       tcredint,ncredint,
       as.mcmc.bugs,lump.mcmc.list,perturb.params,
       HPDregionplot,
       dzinbinom,rzinbinom,
       rchibarsq,dchibarsq,pchibarsq,qchibarsq,
       dmvnorm,dbetabinom,rbetabinom,
       calcslice,gridsearch2d,contour3d,apply2d,curve3d)
## importFrom(rgl,persp3d,lines3d)
importFrom(MASS,kde2d,ginv)
importFrom(coda,mcmc)
importFrom(bbmle,parnames)
importFrom(plyr,aaply)
importFrom(lattice,wireframe)
importFrom("grDevices", "contourLines")
importFrom("graphics", "axTicks", "axis", "contour", "image",
             "locator", "mtext", "par", "persp", "rect")
importFrom("stats", "D", "approx", "dchisq", "dnbinom",
             "na.omit", "pchisq", "plogis", "qchisq", "qlogis", "rbeta",
             "rbinom", "rchisq", "rnbinom", "runif", "uniroot", "var", "coef")
importFrom("utils", "install.packages", "installed.packages")

library(emdbook)
library(R2jags)
library(coda)

data(FirDBHFec_sum)
X <- na.omit(FirDBHFec_sum[,c("fecundity","DBH","pop")])
X$fecundity = round(X$fecundity)
##
DBH <- X$DBH
cones <- X$fecundity
n <- length(DBH)
inits <- list(list(a=0,b=0,k=1),list(a=2,b=-2,k=1),
              list(a=-2,b=2,k=1))
bf <- function(x) system.file("BUGS",x,package="emdbook")
firfec0.bugs <- jags(data=list("DBH","cones","n"),
                     inits,parameters.to.save=c("a","b","k"),
                     model.file=bf("firfec0.bug"),
                     n.chains=length(inits),n.iter=9000)
inits <- list(list(a=c(0,0),b=c(0,0),k=1),
              list(a=c(2,0),b=c(-2,0),k=1),
              list(a=c(0,2),b=c(0,-2),k=1))
grp <- as.numeric(X$WAVE_NON)
firfec1.bugs <- jags(data=list("DBH","cones","n","grp"),
                       inits,parameters.to.save=c("a","b","k"),
                       model.file=bf("firfec1.bug"),
                       n.chains=length(inits),n.iter=6000)
inits <- list(list(a=c(0,0),b=c(0,0),k=c(1,1)),
              list(a=c(2,0),b=c(-2,0),k=c(1,1)),
              list(a=c(0,2),b=c(0,-2),k=c(1,1)))
firfec2.bugs <- jags(data=list("DBH","cones","n","grp"),
                     inits,parameters.to.save=c("a","b","k"),
                     model.file=bf("firfec2.bug"),
                     n.chains=length(inits),n.iter=6000)

save("firfec0.bugs","firfec1.bugs","firfec2.bugs",
     file="firfec-batch3.RData")

is_installed <- function(pkg) {
    pkg %in% installed.packages()[,"Package"]
}

apply2d <-   function(fun,x,y,...,use_plyr=TRUE,.progress="none") {
    if (is.character(fun)) fun <- get(fun)
    if (!use_plyr) {
        a <- apply(expand.grid(x,y),1,function(z) { fun(z[1],z[2],...)})
    } else {
        a <- aaply(as.matrix(expand.grid(x,y)),1,
                   function(z) {fun(z[1],z[2],...)},
                   .progress=.progress)
    }
    return(matrix(a,nrow=length(x)))
}

## FIXME: think about the meaning of ... more carefully
curve3d <- function (expr, from=c(0,0), to=c(1,1),
                     n = c(41,41),
                     xlim, ylim, add = FALSE, 
                     xlab=varnames[1],ylab=varnames[2],
                     zlab = NULL,
                     log = NULL, 
                     sys3d = c("persp","wireframe","rgl","contour","image",
                       "none"),
                     varnames = c("x","y"),
                     use_plyr=TRUE,
                     .progress="none",
                     data = NULL,
                     ...) 
{
    vars <- lapply(as.list(varnames),parse,file="",n=NULL)
    sys3d <- match.arg(sys3d)
    if (add && !(sys3d %in% c("contour","rgl")))
      stop("can only add contour or rgl to 3D plots")
    n = rep(n,length.out=2)
    if (!missing(xlim) && !missing(ylim)) {
      if (!missing(from) | !missing(to)) {
        stop("must specify one of (xlim,ylim) or (from,to)")
      }
      from <- c(xlim[1],ylim[1])
      to <- c(xlim[2],ylim[2])
    }
    sexpr <- substitute(expr)
    if (is.name(sexpr)) {
        fcall <- paste(sexpr, "(",varnames[1],",",varnames[2],")",sep="")
        expr <- parse(text = fcall)
        if (is.null(zlab))
          zlab <- fcall
  } else {
    if (!(is.call(sexpr) && 
          (match(varnames[1], all.vars(sexpr), nomatch = 0) || 
           match(varnames[2], all.vars(sexpr), nomatch = 0))))
      stop(paste("'expr' must be a function or an expression containing '",
           varnames[1],"' and '",varnames[2],"'",sep=""))
    expr <- sexpr
    if (is.null(zlab)) 
      zlab <- deparse(sexpr)
  } 
  lg <- if (length(log)) 
    log
  if (length(lg) == 0) 
    lg <- ""
    x <- if (lg != "" && "x" %in% strsplit(lg, NULL)[[1]]) {
        if (any(c(from[1], to[1]) <= 0)) 
            stop("'from[1]' and 'to[1]' must be > 0 with log=\"x\"")
        exp(seq(log(from[1]), log(to[1]), length = n))
      }  else seq(from[1], to[1], length = n[1])
    y <- if (lg != "" && "y" %in% strsplit(lg, NULL)[[1]]) {
        if (any(c(from[2], to[2]) <= 0)) 
            stop("'from[2]' and 'to[2]' must be > 0 with log=\"y\"")
        exp(seq(log(from[2]), log(to[2]), length = n))
    } else seq(from[2], to[2], length = n[2])
    tmpfun <- function(x,y) {
        env <- c(setNames(list(x,y), varnames), data)
        ## SKIP the inside of curve3d, go back one more level ...
        eval(expr, envir = env, enclos = parent.frame(2))
    }
    z <- apply2d(tmpfun,x,y,use_plyr=use_plyr,.progress=.progress)
    switch(sys3d,
           persp=persp(x,y,z,xlab=xlab,ylab=ylab,zlab=zlab,...),
           contour=contour(x,y,z,xlab=xlab,ylab=ylab,add=add,...),
           image=image(x,y,z,xlab=xlab,ylab=ylab,...),
           none=NA,
           wireframe={
               print(wireframe(z,row.values=x,col.values=y,
                               xlab=xlab,ylab=ylab,zlab=zlab,...))},
           rgl={
              if (!is_installed("rgl")) {
                  stop("please install the rgl package first ...")
              } 
              rgl::persp3d(x,y,z,
                        xlab=xlab,
                        ylab=ylab,zlab=zlab,add=add,...)})
  invisible(list(x=x,y=y,z=z))
}

gridsearch2d <- function(fun,v1min,v2min,v1max,v2max,n1=20,n2=20,
                         logz=FALSE,sys3d=c("both","contour","image"),...) {
  sys3d=match.arg(sys3d)
  redraw = function(v1vec,v2vec,m) {
    if (sys3d=="contour") {
      contour(v1vec,v2vec,m,xlab="",ylab="",...)
    } else image(v1vec,v2vec,m,xlab="",ylab="",...)
    if (sys3d=="both") contour(v1vec,v2vec,m,add=TRUE,...)
  }
  recalc = function(v1vec,v2vec,logz) {
    m = apply2d(fun,v1vec,v2vec)
    mindm = pmax(diff(sort(m)),1e-10)
    if (logz) m = log10(m-min(m)+mindm)
    m
  }
  v1vec = seq(v1min,v1max,length=n1)
  v2vec = seq(v2min,v2max,length=n2)
  m = recalc(v1vec,v2vec,logz)
  stop1 = FALSE
  first = TRUE
  while (!stop1) {
    redraw(v1vec,v2vec,m)
    if (!first) {
      resp = readline("Continue (y|n)? ")
      stop1 = (toupper(resp)=="N")
    }
    first = FALSE
    if (!stop1) {
      cat("click on box corners\n")
      z = lapply(locator(2),sort)
      rect(z$x[1],z$y[1],z$x[2],z$y[2])
      resp = readline("OK (y|n)? ")
      if (toupper(resp)=="Y") {
        v1min=z$x[1]
        v1max=z$x[2]
        v2min=z$y[1]
        v2max=z$y[2]
        v1vec = seq(v1min,v1max,length=n1)
        v2vec = seq(v2min,v2max,length=n2)
        m = recalc(v1vec,v2vec,logz)
      }
    }
  }
  resp = readline("get point (y|n)? ")
  if (toupper(resp)=="Y") {
    cat("click on point\n")
    z=locator(1)
    return(z)
  } else invisible(NULL)
}

metropSB <- function(fn,start,deltap=NULL,
                      scale=1,rptfreq=-1,
                      acceptscale=1.01,rejectscale=0.99,
                      nmax=10000,retvals=FALSE,retfreq=100,
                      verbose=FALSE,...) {
  ## initialization
  ndim <- length(start)  # number of parameters/dimensions
  p <- start             # current parameter vector = starting vector
  minp <- start          # parameter vector corresponding to minimum value so far
  val <- fn(p,...)     # current function value
  minval <- val          # minimum value so far
  if (retvals) {
    info <- matrix(nrow=round(nmax/retfreq),ncol=3*ndim+3)
    dimnames(info) <- list(NULL,c(paste("p",1:ndim,sep=""),
                                  paste("minp",1:ndim,sep=""),
                                  paste("deltap",1:ndim,sep=""),
                                        "val","minval","accept"))
    # save info on parameter values, function value, whether accepted or not, current best parameters,
    # current best value, current jump size
  }
  it <- 1                # iteration count
  if (is.null(deltap)) deltap <- p*0.05   # default starting jump size(s)
  while (it<=nmax) {
    oldp <- p                             # save current parameters and function value
    oldval <- val
    p <- p + runif(ndim,-1,1)*deltap      # perturb current values
    val <- fn(p,...)                    # new function value
    dval <- val-oldval                    # change
    saveinfo <- (retvals && it %% retfreq == 0)
    savecount <- it %/% retfreq
    if (saveinfo) {
      info[savecount,-ncol(info)] <- c(p,minp,deltap,val,minval)
    }
    if (verbose)
      if (it %% 100 == 0)
         cat(it,"\n")
    if (dval<0 || (exp(-dval*scale)>runif(1))) {
      ## accept new value
      if (saveinfo) info[savecount,ncol(info)] <- 1
      if (val<minval) {  # if better than best so far, record new minimum
	minval <- val
	minp <- p
      }
      deltap <- deltap*acceptscale   # increase jump size
    } else {
      ## replace old values
      if (saveinfo) info[savecount,ncol(info)] <- 0
      val <- oldval
      p <- oldp
      deltap <- deltap*rejectscale   # decrease jump size
    }
    it <- it+1
    if ((rptfreq>0) && (it %% rptfreq)==1)
      cat("it=",it," value=",val," min. val.=",minval,"\n")
  }
  if (retvals)
    return(list(minimum=minval,estimate=minp,funcalls=it,retvals=info))
  else
    return(list(minimum=minval,estimate=minp,funcalls=it))
}

contour3d <- function(x,y,z,contourArgs=NULL,...) {
    if (!is_installed("rgl")) {
        stop("must install the rgl package to use contour3d")
    }
  if (is.list(x)) {
    if (!all(sort(names(x))==c("x","y","z")))
      stop("list should contain components 'x', 'y', 'z'")
    z <- x$z
    y <- x$y
    x <- x$x
  }
  ccc = do.call(contourLines,c(list(x,y,z),contourArgs))
  ccc = lapply(ccc, function(x) {
    list(x=x$x,y=x$y,z=rep(x$level,length(x$x))) })
  invisible(mapply(rgl::lines3d,ccc,MoreArgs=list(...)))
  invisible(ccc)
}

rchibarsq <- function(n,df=1,mix=0.5) {
  ifelse(runif(n)>mix,
         rchisq(n,df),
         if (df==1) 0 else rchisq(n,df-1))
}

dchibarsq <- function(x,df=1,mix=0.5,log=FALSE) {
  df <- rep(df,length.out=length(x))
  mix <- rep(mix,length.out=length(x))
  c1 <- ifelse(df==1,0,dchisq(x,df-1))
  c2 <- dchisq(x,df)
  r <- mix*c1+(1-mix)*c2
  zeros <- (x==0 & df==1)
  if (any(zeros)) {
    r[zeros] <- Inf
  }
  if (log) log(r) else r
}

pchibarsq <- function(p,df=1,mix=0.5,lower.tail=TRUE,log.p=FALSE) {
  df <- rep(df,length.out=length(p))
  mix <- rep(mix,length.out=length(p))
  c1 <- ifelse(df==1,if (lower.tail) 1 else 0,
               pchisq(p,df-1,lower.tail=lower.tail))
  c2 <- pchisq(p,df,lower.tail=lower.tail)
  r <- mix*c1+(1-mix)*c2
  if (log.p) log(r) else r
}

qchibarsq <- function(q,df=1,mix=0.5) {
  n <- max(length(q),length(df),length(mix))
  df <- rep(df,length.out=n)
  mix <- rep(mix,length.out=n)
  q <- rep(q,length.out=n)
  tmpf2 <- function(q,df,mix) {
    if (df>1) {
      tmpf <- function(x) {
        pchibarsq(x,df,mix)-q
      }
      uniroot(tmpf,lower=qchisq(q,df-1),upper=qchisq(q,df))$root
    } else {
      newq <- (q-mix)/(1-mix)
      ifelse(newq<0,0,qchisq(newq,df=1))
    }
  }
  mapply(tmpf2,q,df,mix)
  ##  if (any(df>1)) stop("df>1 not implemented yet")
  ## ?? need uniroot() solution to find quantiles of
  ##   mixtures?
}

dmvnorm <- function (x, mu, Sigma, log = FALSE, tol = 1e-06) {
    if (is.vector(x)) 
        x = t(as.matrix(x))
    n = length(mu)
    if (is.vector(mu)) {
        p <- length(mu)
        if (is.matrix(x)) {
            mu <- matrix(rep(mu, nrow(x)), ncol = p, byrow = TRUE)
        }
    }
    else {
        p <- ncol(mu)
    }
    if (!all(dim(Sigma) == c(p, p)) || nrow(x) != nrow(mu)) 
        stop("incompatible arguments")
    eS <- eigen(Sigma, symmetric = TRUE) 
    ev <- eS$values
    if (!all(ev >= -tol * abs(ev[1]))) 
        stop("Sigma is not positive definite")
    z <- t(x - mu)
    logdetS <- try(determinant(Sigma, logarithm = TRUE)$modulus,
                   silent=TRUE)
    attributes(logdetS) <- NULL
    iS <- MASS::ginv(Sigma)
    ssq <- diag(t(z) %*% iS %*% z)
    loglik <- -(n * (log(2*pi)) +  logdetS + ssq)/2
    if (log) return(loglik) else return(exp(loglik))
  }


nonint <- function(x) (abs((x) - floor((x)+0.5)) > 1e-7)

dbetabinom <- function(x,prob,size,theta,shape1,shape2,log=FALSE) {
  if (missing(prob) && !missing(shape1) && !missing(shape2)) {
    prob <- shape1/(shape1+shape2)
    theta <- shape1+shape2
  }
  v <- lfactorial(size)-lfactorial(x)-lfactorial(size-x)-
      lbeta(theta*(1-prob),theta*prob)+lbeta(size-x+theta*(1-prob),x+theta*prob)
  if (any(n <- nonint(x))) {
      warning("non-integer x detected; returning zero probability")
      v[n] <- -Inf
  }
  if (log) v else exp(v)
}

rbetabinom <- function(n,prob,size,theta,shape1,shape2) {
  if (!missing(prob) && !missing(size) && missing(shape1) && missing(shape2)) {
    shape1 <- theta*prob
    shape2 <- theta*(1-prob)
  }
  rbinom(n,size=size,prob=rbeta(n,shape1,shape2))
}

## could implement distribution function as:
## D(x) = 1 - (n B(b+n-x-1,a+x+1) Gamma(n) 3_F_2(1,a+x+1,-n+x+1;
##   x+2, -b-n+x+2;1))/(B(a,b) B(n-x,x+2) Gamma(n+2))
## (from Mathworld) -- would need to get 3F2 somewhere?
## not in GSL!
## http://tolstoy.newcastle.edu.au/R/e2/help/07/02/10988.html
##  suggests Davies package could be adapted
##  or ?? SuppDists ??

## zero-inflated negative binomial

dzinbinom <- function(x,mu,size,zprob,log=FALSE) {
  logv = log(1-zprob) + dnbinom(x,mu=mu,size=size,log=TRUE)
  logv = ifelse(x==0,log(zprob+exp(logv)),logv)
  if (log) logv else exp(logv)
}

rzinbinom <- function(n,mu,size,zprob) {
  ifelse(runif(n)<zprob,0,rnbinom(n,mu=mu,size=size))
}

## add q, p functions for zinbinom?  where else is zinbinom
## implemented?

## from <http://mathworld.wolfram.com/LambertW-Function.html>,
## ultimately from Corless et al 1996
lWasymp <- function(z,logz) {
  if (!missing(z) && any(!is.finite(log(z))))
    stop("overflow error: please specify logz")
  L1 <- if (!missing(logz)) logz else log(z)
  L2 <- log(logz)
  L1-L2+L2/L1+L2*(L2-2)/(2*L1^2)+L2*(6-9*L2+2*L2^2)/(6*L1^3)+
    L2*(-12+36*L2-22*L2^2+3*L2^3)/(12*L1^4)+L2*(60-300*L2+350*L2^2-125*L2^3+12*L2^4)/(60*L1^5)
}


lambertW <- function(z,...) {
  bigz <- Mod(z)>1e307
  if (!any(na.omit(bigz))) return(lambertW_base(z,...))
  res <- rep(NA_real_,length(z))
  res[bigz] <- lWasymp(logz=log(z[bigz]))
  res[!bigz] <- lambertW_base(z[!bigz],...)
  res
}

lambertW_base <- function(z,b=0,maxiter=10,eps=.Machine$double.eps,min.imag=1e-9) {
  badz <- !is.finite(z)
  z.old <- z
  z <- z[!badz]
  if (any(round(Re(b)) != b))
    stop("branch number for W must be an integer")
  if (!is.complex(z) && any(z<0)) z <- as.complex(z)
  ## series expansion about -1/e
  ##
  ## p = (1 - 2*abs(b)).*sqrt(2*e*z + 2);
  ## w = (11/72)*p;
  ## w = (w - 1/3).*p;
  ## w = (w + 1).*p - 1
  ##
  ## first-order version suffices:
  ##
  w = (1 - 2*abs(b))*sqrt(2*exp(1)*z + 2) - 1
  ## asymptotic expansion at 0 and Inf
  ##
  v = log(z + as.numeric(z==0 & b==0)) + 2*pi*b*1i;
  v = v - log(v + as.numeric(v==0))
  ## choose strategy for initial guess
  ##
  c = abs(z + exp(-1))
  c = (c > 1.45 - 1.1*abs(b))
  c = c | (b*Im(z) > 0) | (!Im(z) & (b == 1))
  w = (1 - c)*w + c*v
  ## Halley iteration
  ##
  for (n in 1:maxiter) {
    p = exp(w)
    t = w*p - z
    f = (w != -1)
    t = f*t/(p*(w + f) - 0.5*(w + 2.0)*t/(w + f))
    w = w - t
    ## if (any(is.na(t) | is.na(w))) {
    ## FIXME: what to do here?
    ## stop("NAs encountered in LambertW")
    ## print(t)
    ## print(w)
    ## }
    ok <- is.finite(t) & is.finite(w)
    ## iterate until ALL values converged -- perhaps
    ## inefficient
    if (all(abs(Re(t[ok])) < (2.48*eps)*(1.0 + abs(Re(w[ok])))
            & abs(Im(t[ok])) < (2.48*eps)*(1.0 + abs(Im(w[ok])))))
      break
  }
  if (n==maxiter) warning(
                      sprintf("iteration limit (%d) reached, result of W may be inaccurate (z=%f)",
                              maxiter,z))
  if (all(abs(Im(w[!is.na(w)]))<min.imag)) w <- as.numeric(w)
  if (sum(badz)>0) {
    w.new <- rep(NA_real_,length(z.old))
    w.new[!badz] <- w
    w <- w.new
  }
  return(w)
}

##
## these are OBSOLETE:  see xyplot (not traceplot) method in coda
##
## new functions for operating on Bayesian stuff
## traceplot <- function (x, smooth=TRUE, ylab="", sys="lattice", ...) {
##     UseMethod("traceplot")
##   }

## traceplot.mcmc.list <- function(x,smooth=TRUE,ylab="", sys="lattice", ...) {
##   require(reshape)
##   require(lattice)
##   nchains = length(x)
##   niter = nrow(x[[1]])
##   x2 = melt(data.frame(do.call(rbind,x),
##     iter=rep(1:niter,nchains),
##     chain=rep(1:nchains,each=niter)),
##     id.var=c("chain","iter"))
##   type <- "l"
##   if (smooth) type <- c(type,"smooth")
##   xyplot(value~iter|variable,groups=chain,type=type,data=x2,
##          scales=list(y=list(relation="free")),
##          ylab=ylab,xlab="iteration")
## }

## traceplot.mcmc <- function(x,smooth=TRUE,ylab="", sys="lattice", ...) {
##   require(reshape)
##   require(lattice)
##   niter = nrow(x)
##   x2 = melt(data.frame(x,
##     iter=1:niter),
##     id.var="iter")
##   type <- "l"
##   if (smooth) type <- c(type,"smooth")
##   xyplot(value~iter|variable,type=type,data=x2,
##          scales=list(y=list(relation="free")),
##          ylab=ylab,xlab="iteration")
## }

## copied from traceplot in coda package, but uses lattice
## traceplot.mcmc <- function (x, data = NULL, outer, aspect = "xy",
##                             default.scales = list(y=list(relation = "free")), 
##                             start = 1, thin = 1,
##                             main = attr(x, "title"), xlab = "",
##                             ylab="",
##                             plot.points = "rug", ...,
##                             subset = coda:::thinned.indices(x, start = start, 
##                               thin = thin))
##   ## would like to supply subset argument that would also
##   ## select specific variables
##   ## consider aspect="fill" as an alternative
## {
##     require(lattice)
##     if (!is.R()) {
##         stop("This function is not yet available in S-PLUS")
##     }
##     if (!missing(outer)) 
##         warning("specification of outer ignored")
##     data <- as.data.frame(x)
##     v <- seq(nrow(x))
##     form <- as.formula(paste(paste(lapply(names(data), as.name),
##         collapse = "+"), "~v"))
##     xyplot(form, data = data[subset, ], outer = TRUE, aspect = aspect, 
##            default.scales = default.scales, main = main, xlab = xlab,
##            ylab=ylab,
##            type="l",
##            plot.points = plot.points, ...)
## }

HPDregionplot <- function(x,vars=1:2,h,n=50,lump=TRUE,prob=0.95,
                          xlab=NULL,ylab=NULL,lims=NULL,...) {
  parnames <- if (inherits(x, "mcmc.list")) colnames(x[[1]]) else colnames(x)
  if (is.character(vars)) {
    vars <- match(vars,parnames)
    if (any(is.na(vars))) stop("some variable names do not match")
  }
  varnames <- parnames[vars]
  mult <- (class(x)=="mcmc.list" && !lump)
  if (mult) stop("multiple chains without lumping not yet implemented")
  if (inherits(x, "mcmc.list")) {
    if (lump) var1 <- c(sapply(x,function(z)z[,vars[1]]))
    else var1 <- lapply(x,function(z)z[,vars[1]])
  } else var1 <- x[,vars[1]]
  if (inherits(x,"mcmc.list")) {
    if (lump) var2 <- c(sapply(x,function(z)z[,vars[2]]))
    else var2 <- lapply(x,function(z)z[,vars[2]])
  } else var2 <- x[,vars[2]]
  if (is.null(lims)) lims <- c(range(var1),range(var2))
  if (!mult) {
    post1 <- kde2d(var1,var2,n=n,h=h,lims=lims)
    ## post0 = post1
  } else {
    post1 = mapply(kde2d,var1,var2,MoreArgs=list(n=n,h=h,lims=lims))
  }
  dx <- diff(post1$x[1:2])
  dy <- diff(post1$y[1:2])
  sz <- sort(post1$z)
  ## debugging stuff
  ## if (FALSE) {
  ##   lattice:::contourplot(post1$z)
  ##   d <- with(post1,data.frame(expand.grid(y=y,x=x),z=c(z)))
  ##   lattice:::contourplot(z~x*y,data=d)
  ##   with(post1,contour(x,y,z))
  ##   points(x[,1],x[,2],col=2)
  ##   afun <- function(n) {
  ##     k2 <- kde2d(x[,1],x[,2],n=n,h=c(1,1))
  ##     with(k2,sum(z)*diff(x)[1]*diff(y)[1])
  ##   }
  ##   sapply(5:25,afun)
  ## }
  c1 <- cumsum(sz)*dx*dy
  ## trying to find level containing 95% of volume ...
  levels <- sapply(prob,function(x) {approx(c1,sz,xout=1-x)$y})
  ## meanvec <- c(mean(var1),mean(var2))
  if (is.null(xlab)) xlab <- varnames[1]
  if (is.null(ylab)) ylab <- varnames[2]
  contour(post1$x,post1$y,post1$z,level=levels,
          xlab=xlab,ylab=ylab,drawlabels=FALSE,...)
  invisible(contourLines(post1$x,post1$y,post1$z,levels=levels))
}

## make an mcmc object out of an mcmc.list
lump.mcmc.list <- function(x) {
  x2 <- do.call("rbind",x)
  mcpars <- sapply(x,attr,"mcpar")
  class(x2) <- "mcmc"
  if (var(mcpars[1,])>0 || var(mcpars[3,])>0)
    stop("can't combine chains with unequal start/thin")
  attr(x2,"mcpar") <- c(mcpars[1,1],sum(mcpars[2,]),mcpars[3,1])
  x2
}

perturb.params = function(base,alt,which,
  mult=FALSE,use.base=TRUE) {
  if (!missing(alt)) {
    chlist = mapply(
      function(name,vals) {
        x = lapply(vals,
               function(z2) {
                 if (mult) {
                   base[[name]] = base[[name]]*z2}
                 else base[[name]]=z2
                 base
               })
        if (is.list(x)) x else list(x)
      },
      names(alt),alt)
    chlist = unlist(chlist,recursive=FALSE)
  }
  if (use.base) {
    chlist=append(list(base),chlist)
  }
  chlist
}

get.emdbook.packages <- function() {
   pkglist = c(## "adapt",
     "bbmle","chron",
     "coda","ellipse","ggplot2",
     "gplots","gtools","gdata",
     "MCMCpack","deSolve","plotrix","reshape","rgl",
     "scatterplot3d")
   inst.pkgs = rownames(installed.packages())
   newpkgs <- pkglist[!pkglist %in% inst.pkgs]
   if (length(newpkgs)>0) {
      install.packages(newpkgs)
   ##  do.call("install.packages",list(pkglist))
   ## sapply(pkglist,install.packages)
   }
   warning("The adapt package is no longer available. You can work through 99% of the material in _Ecological Models and Data_ without it; for more information see http://emdbolker.wikidot.com/r")
 }


## expr: expression to evaluate (raw form)
## meanval: values of the mean (possibly named)
## vars: names
## Sigma: var-cov function
deltavar <- function(fun,meanval=NULL,vars,Sigma,verbose=FALSE
                     ##, symbderivs=NULL
                     ) {
  expr <- as.expression(substitute(fun))
  nvals <- length(eval(expr,envir=as.list(meanval)))
  vecexp <- nvals>1 ## is the result a vector?
  if (missing(vars)) {
    if (missing(meanval) || is.null(names(meanval)))
      stop("must specify either variable names or named values for means")
    vars <- names(meanval)
  }
  ## STOPPED HERE
  derivs <- try(lapply(vars,D,expr=expr),silent=TRUE)
  ## symbderivs <- TRUE
  if (inherits(derivs,"try-error")) {
      w <- NULL
      if (!Sys.getenv("LANGUAGE") %in% c("", "en")) {
          w <- "error (symbols not in derivative table?): using numeric derivatives"
      } else if (any(grepl("derivatives table", derivs))) {
          w <- "some symbols not in derivative table, using numeric derivatives"
      }
      if (!is.null(w)) {
          ## take numeric derivative
          symbderivs <- FALSE
          warning(w)
          nderivs <- with(as.list(meanval),
                          numericDeriv(expr[[1]],theta=vars))
          nderivs <- attr(nderivs,"gradient")
      } else {
          stop(paste("Error within derivs:",derivs))
      }
  } else {
      nderivs <- sapply(derivs,eval,envir=as.list(meanval))
  }
  if (verbose) {
      ## if (symbderivs) {
      ##     stop("symbolic derivs not yet implemented\n")
      ##     cat("symbolic derivs:\n")
      ##     print(derivs)
      ## }
      cat("value of derivs:\n")
      print(nderivs)
  }
  if (!is.matrix(Sigma) && length(Sigma)>1) Sigma <- diag(Sigma)
  ## if (!is.matrix(Sigma)) sum(Sigma*nderivs^2) else
  if (vecexp && is.list(nderivs)) nderivs <- do.call("cbind",nderivs)
  if (is.matrix(nderivs)) {
    r <- apply(nderivs,1,function(z) c(z %*% Sigma %*% matrix(z)))
  } else r <- c(nderivs %*% Sigma %*% matrix(nderivs))
  ## really only want diagonal
  r
}

deltamethod <- function(fun,z,var="x",params=NULL,max.order=2) {
  d0 <- as.expression(substitute(fun))
  dvals <- list()
  dvals[[1]] <- d0
  for (i in 2:(max.order+1)) {
    dvals[[i]] <- D(dvals[[i-1]],var)
  }
  mvals <- numeric(max.order-1)
  m <- mean(z)
  for (i in 1:(max.order-1)) {
    mvals[i] <- mean((z-m)^(i+1))
  }
  mvals[1] <- var(z)  ## kluge
  ev1 <- c(as.list(params),list(m))
  ev2 <- c(as.list(params),list(z))
  names(ev1)[length(ev1)] <-   names(ev2)[length(ev2)] <- var
  r0 <- mean(eval(d0,ev2)) ## true value
  evals <- sapply(dvals,eval,ev1)  ## evaluated derivatives
  gvals <- gamma(c(1,3:(max.order+1)))
  deltavals <- cumsum(c(1,mvals)*c(evals[-(2)])/gvals)
  results <- c(r0,deltavals)
  names(results) = c("E(f(x)) [true]","f(E(x)) [naive]",paste("delta",2:max.order,sep=""))
  results
}

lseq <- function(from,to,length.out) {
  exp(seq(log(from),log(to),length.out=length.out))
}

## utility function for formatting
scinot <- function(x,format=c("latex","expression"),delim="$",
                   pref="",...) {
  format <- match.arg(format)
  y <- strsplit(as.character(formatC(x,format="e",...)),"e")[[1]]
  y[1] <- gsub("^0+","",y[1])
  y[2] <- ifelse(length(grep("^\\+",y[2]))>0,
                 gsub("^\\+0+","",y[2]),
                 gsub("^-0+","-",y[2]))
  if (format=="latex") { 
    v <- paste(delim,y[1],"\\\\times 10^{",y[2],"}",delim,sep="")
  } else if (format=="expression") {
    if (as.numeric(y[1])==1) {
      v <- substitute(expression(paste(pref,10^b)),list(pref=pref,b=as.numeric(y[2])))
    } else {
      v <- substitute(expression(paste(pref,a %*% 10^b)),
                      list(pref=pref,a=as.numeric(y[1]),b=as.numeric(y[2])))
    }
  }
  v
}

## axis in scientific notation/expressions
## obsolete via sfsmisc?
axis.scinot <- function(side,at) {
  if (missing(at)) at <- axTicks(side)
  axis(side=side,labels=FALSE)
  invisible(lapply(at,
         function(a) mtext(side=side,at=a,eval(scinot(a,"expression")),
                           line=par("mgp")[2])))
}

## convert R2WinBUGS output to coda/mcmc
as.mcmc.bugs <- function(x) {
  if (x$n.chains>1) {
    z <- list()
    for (i in 1:x$n.chains) {
      z[[i]] <- mcmc(x$sims.array[,i,],start=1,thin=x$n.thin)
    }
    class(z) <- "mcmc.list"
  } else {
    z <- mcmc(x$sims.matrix,start=1,thin=x$n.thin)
  } 
  return(z)
}

## credible interval for a theoretical distribution
tcredint <- function(dist,parlist,ranges,level=0.95,eps=1e-5,verbose=FALSE) {
  qfun = function(x) do.call(paste("q",dist,sep=""),c(list(x),parlist))
  dfun = function(x) do.call(paste("d",dist,sep=""),c(list(x),parlist))
  pfun = function(x) do.call(paste("p",dist,sep=""),c(list(x),parlist))
  if (missing(ranges))  ## set upper/lower limits for search by quantiles
    ranges <- qfun(c(eps,0.5,1-eps))
  lims <- function(pdens) { ## find lower and upper values for which prob dens = target value
    lower <- uniroot(function(p) {dfun(p)-pdens},
                     interval=ranges[1:2])$root
    upper <- uniroot(function(p) {dfun(p)-pdens},
                     interval=ranges[2:3])$root
    c(lower,upper)
  }
  limarea <- function(pdens) { ## find area between target values
     intlim <- lims(pdens)
     d <- diff(pfun(intlim))
     ## cat(pdens,intlim,d,"\n")
     d
   }
  ## these limits must be within limits set above
  v1 <- qfun(c(0.6,0.9999)) # quantiles
  v2 <- dfun(v1)  ## bracketing densities
  u <- uniroot(function(x) {limarea(x)-level},
                 interval=v2)
  intlim <- lims(u$root)
  r = c(intlim,dfun(intlim[1]),limarea(u$root))
  names(r) = c("lower","upper","p","area")
  if (verbose) r else r[1:2]
}

## credible interval for (1D) posterior distribution stored in a numeric
## vector.  assumed unimodal!  pvec (vector of parameter values),
## npost (vector of posterior densities), level, tolerance
ncredint <- function(pvec,npost,level=0.95,tol=0.01,verbose=FALSE) {
  dx = diff(pvec)[1]
  cumdist <- cumsum(npost)*dx
  midpt <- which.min(abs(cumdist-0.5))
  lims <- function(pdens) { ## find lower and upper values for which
    ## prob dens is closest to target value
    lower <- which.min(abs(npost[1:midpt]-pdens))
    upper <- which.min(abs(npost[(midpt+1):length(npost)]-pdens))+midpt
    c(lower,upper)
  }
  limarea <- function(pdens) {
    intlim <- lims(pdens)
    d <- sum(npost[intlim[1]:intlim[2]])*dx
    ##    cat(pdens,intlim,d,"\n")
    d
  }
  ## find credible interval
  v2 <- seq(0,max(npost),by=tol)
  vals <- sapply(v2,limarea)
  w <- which.min(abs(vals-level))
  r = c(pvec[lims(v2[w])],v2[w],limarea(v2[w]))
  names(r) = c("lower","upper","p","area")
  ## credible intervals; posterior density, area
  if (verbose) return(r) else return(r[1:2])
}



calcslice <- function(fit1,fit2,fn=fit1@minuslogl,
                      range=c(-0.1,1.1),
                      n=400) {
  slicep = seq(range[1],range[2],length=n)
  slicepars = t(sapply(slicep,function(x) (1-x)*coef(fit1)+x*coef(fit2)))
  ## FIXME: warning about parnames from R CMD check
  if (is.null(parnames(fn))) {
    dd <- fit1@data
    ff <- names(formals(fn))
    if (!("..." %in% ff)) {
      dd <- dd[names(dd) %in% ff]
    }
    v = apply(slicepars,1,function(x) do.call(fn,c(as.list(x,dd))))
  } else { ## vector-argument function
    v = apply(slicepars,1,fn)
  }
  list(x=slicep,y=v)
}


trcoef <- function(x,inverse=FALSE) {
  ## n.b. logit should come before log
  vals=list(pattern=c("logit","log","sqrt"),
    fun=c(plogis,exp,function(x) {x^2}),
    ifun=c(qlogis,log,sqrt))
  if (is.null(attr(x,"transf"))) attr(x,"transf") = character(length(x))
  for (p in 1:length(vals[[1]])) {
    pat = vals$pattern[[p]]
    fun = vals$fun[[p]]
    ifun = vals$ifun[[p]]
    if (!inverse) {
      w = grep(paste("^",pat,sep=""),names(x))
      if (length(w)>0) {
        attr(x,"transf")[w]=pat
        names(x)[w] = gsub(pat,"",names(x)[w])      
        x[w] = sapply(x[w],fun)
      }
    } else {
      w = which(attr(x,"transf")==pat)
      if (length(w)>0) {
        attr(x,"transf")[w] = ""
        names(x)[w] = paste(pat,names(x)[w],sep="")
        x[w] = sapply(x[w],ifun)
      }
    }
  }
  if (all(attr(x,"transf")=="")) attr(x,"transf") = NULL
  x
}

library(testthat)
library(emdbook)
fakegamma <- function(...) gamma(...)

## to test whether
test_msg <- function(msg,silent=FALSE) {
   grepl(msg,try(x ,silent=silent))
}
french_msg <- function() {
    test_msg("introuvable", silent=TRUE)
}
context("deltavar",

        test_that("numeric derivs fallback", {
            Sys.setlocale("LC_MESSAGES", "C")
            m <- c(scale=0.8,shape=12)
            S <- matrix(c(0.015,0.125,0.125,8.97), nrow=2)
            expect_warning(d1 <- deltavar(scale*fakegamma(1+1/shape),
                                  meanval=m,
                                  Sigma=S))
            Sys.setenv(

\name{dbetabinom}
\alias{BetaBinomial}
\alias{dbetabinom}
\alias{rbetabinom}
\title{Beta-binomial distribution}
\description{
   Density function and random variate generator 
for the beta-binomial function, parameterized in
terms of probability and overdispersion
}
\usage{
dbetabinom(x, prob, size,  theta, shape1, shape2, log = FALSE)
rbetabinom(n, prob, size, theta, shape1, shape2)
}
\arguments{
  \item{x}{a numeric vector of integer values}
  \item{prob}{numeric vector: mean probability of underlying beta distribution}
  \item{size}{integer: number of samples}
  \item{theta}{overdispersion parameter}
  \item{shape1}{shape parameter of per-trial probability distribution}
  \item{shape2}{shape parameter of per-trial probability distribution}
  \item{log}{(logical) return log probability density?} 
  \item{n}{integer number of random variates to return}
}
\details{
   The beta-binomial distribution is the result of
compounding a beta distribution of probabilities with
a binomial sampling process.  The density function is
\deqn{p(x) = \frac{C(N,x) \mbox{Beta}(x+\theta p,N-x+\theta(1-p))}%
{\mbox{Beta}(\theta p,\theta(1-p))}%
}{p(x) = (C(N,x)*Beta(x+theta*p,N-x+theta*(1-p)))/Beta(theta*p,theta*(1-p))}
The parameters \code{shape1} and \code{shape2} are
the more traditional parameterization in terms of
the parameters of the per-trial probability distribution.
}
\value{
  A vector of probability densities or random deviates.
  If \code{x} is non-integer, the result is zero (and
  a warning is given).
}
\references{Morris (1997), American Naturalist 150:299-327;
\url{https://en.wikipedia.org/wiki/Beta-binomial_distribution}}
\note{Although the quantile (qbetabinom)
  and cumulative distribution (pbetabinom)
  functions are not available, in a pinch they
  could be computed from the \code{pghyper} and
  \code{qghyper} functions in the \code{SuppDists}
  package -- provided that \code{shape2>1}.  As
  described in \code{?pghyper}, \code{pghyper(q,a=-shape1,
    N=-shape1-shape2,k=size)} should give the
  cumulative distribution for the beta-binomial
  distribution with parameters (shape1,shape2,size),
  and similarly for \code{qghyper}.
  (Translation to the (theta,size,prob) parameterization
  is left as an exercise.)}
\author{Ben Bolker}
\seealso{\code{\link{dbeta}}, \code{\link{dbinom}}}
\examples{
  set.seed(100)
  n <- 9
  z <- rbetabinom(1000, 0.5, size=n, theta=4)
  par(las=1,bty="l")
  plot(table(z)/length(z),ylim=c(0,0.34),col="gray",lwd=4,
       ylab="probability")
  points(0:n,dbinom(0:n,size=n,prob=0.5),col=2,pch=16,type="b")
  points(0:n,dbetabinom(0:n,size=n,theta=4,
           prob=0.5),col=4,pch=17,type="b")
  ## correspondence with SuppDists 
  if (require(SuppDists)) {
    d1a <- dghyper(0:5,a=-5,N=-10,k=5)
    d1b <- dbetabinom(0:5,shape1=5,shape2=5,size=5)
    max(abs(d1a-d1b))
    p1a <- pghyper(0:5,a=-5,N=-10,k=5,lower.tail=TRUE)
    p1b <- cumsum(d1b)
    max(abs(p1a-p1b))
  } 
}
\keyword{distribution}

\name{SeedPred}
\alias{SeedPred}
\alias{SeedPred_wide}
\alias{SeedPred_mass}
\docType{data}
\title{Seed predation data set from Duncan and Duncan 2000}
\description{
   Data on seed predation over time from Duncan and Duncan (2000)
}
\usage{
data(SeedPred)
data(SeedPred_wide)
data(SeedPred_mass)
}
\format{
  \describe{
    \item{\code{station}}{a factor specifying the station number}
    \item{\code{species}}{a factor with levels \code{abz} \code{cd} \code{cor} \code{dio} \code{mmu} \code{pol} \code{psd} \code{uva}}
    \item{\code{date}}{sample date}
    \item{\code{seeds}}{number of seeds present}
    \item{\code{tcum}}{cumulative time elapsed}
    \item{\code{tint}}{time since last sample}
    \item{\code{taken}}{seeds removed since last sample}
    \item{\code{dist}}{distance from forest edge (m)}
  }
}
\details{
  \code{SeedPred} is in long format, \code{SeedPred_wide} is in wide
  format; \code{SeedPred_wide} has lots of \code{NA} values because
  stations at 10 and 25 m from the forest were sampled on different
  days. \code{SeedPred_mass} is a numeric vector containing the
  approximate seed masses for each species.
  }
\source{
   R. Scot Duncan and Virginia E. Duncan (2000) Forest Succession and Distance from Forest Edge in an Afro-Tropical Grassland, Biotropica 32(1):33-41. (Data from Scot Duncan.)
}
\examples{
data(SeedPred)
}
\keyword{datasets}

\name{calcslice}
\alias{calcslice}
\title{Negative log-likelihood slice}
\description{
  Calculate the negative log-likelihood along a line
  connecting two \code{mle} fits
}
\usage{
calcslice(fit1, fit2, fn = fit1@minuslogl, range = c(-0.1, 1.1), n = 400)
}
\arguments{
  \item{fit1}{An \code{mle} object}
  \item{fit2}{Another \code{mle} object}
  \item{fn}{Negative log-likelihood function}
  \item{range}{Numeric vector: range of parameters to try, where
    0 corresponds to \code{coef(fit1)} and 1 corresponds to \code{coef(fit2)}}
  \item{n}{Number of points to evaluate}
}
\details{
  Calculates the negative log-likelihood (not a profile, just a
  "slice") along the line connecting the two sets of coefficients.
  Intended for diagnosing and visualizing multiple minima in
  a likelihood surface, especially in higher-dimensional models.
}
\value{
  \item{x}{Parameter values, along the 0-1 scale described above}
  \item{y}{Negative log-likelihood values}
}
\author{Ben Bolker}
\keyword{misc}

\name{metropSB}
\alias{metropSB}
\title{Metropolis-Szymura-Barton algorithm}
\description{
  Stochastic global optimization using the Metropolis-Szymura-Barton
  algorithm. New parameters are chosen from a uniform candidate
  distribution with an adaptively tuned scale, and accepted or rejected
  according to a Metropolis rule.
}
\usage{
metropSB(fn, start, deltap = NULL, scale = 1, rptfreq = -1, acceptscale
= 1.01, rejectscale = 0.99, nmax = 10000,
retvals = FALSE, retfreq = 100, verbose = FALSE, ...)
}
\arguments{
  \item{fn}{Objective function, taking a vector of parameters as its
    first argument.  The function is minimized, so it should be a
    negative log-likelihood or a negative log-posterior density.}
  \item{start}{Vector of starting values}
  \item{deltap}{Starting jump size; half-width of uniform distribution}
  \item{scale}{Scaling factor for acceptance}
  \item{rptfreq}{Frequency for reporting interim results (<0 means no reporting)}
  \item{acceptscale}{Amount to inflate candidate distribution if last
    jump was accepted}
  \item{rejectscale}{Amount to shrink candidate distribution if last
    jump was rejected}
  \item{nmax}{Number of iterations}
  \item{retvals}{Return detailed statistics?}
  \item{retfreq}{Sampling frequency for detailed statistics}
  \item{verbose}{Print status?}
  \item{\dots}{Other arguments to \code{fn}}
}
\details{
  Metropolis-Szymura-Barton algorithm: given function and starting value, try to find
  parameters that minimize the function
 Algorithm: at a given step,
 1. pick a new set of parameters, each of which is uniformly distributed
    in (p[i]-deltap[i],p[i]+deltap[i])
 2. calculate function value at new parameter values
 3. if f(new)<f(old), accept
 4. if f(new)>f(old), accept with probability (exp(-scale*(f(new)-f(old)))
 5. if accept, increase all deltap values by acceptscale; if reject, decrease by rejectscale
 6. if better than min so far, save function and parameter values
 7. if reject, restore old values
}
\value{
  \item{minimum}{minimum value achieved}
  \item{estimate}{parameters corresponding to minimum}
  \item{funcalls}{number of function evaluations}
  If \code{retvals=TRUE}:
  \item{retvals}{matrix of periodic samples including parameters,
    jump scale, current value, and minimum achieved value}
}
\references{Szymura and Barton (1986) Genetic analysis of a hybrid zone
  between the fire-bellied toads,Bombina bombina and B. variegata, near
  Cracow in southern Poland. Evolution 40(6):1141-1159.
}
\note{  If \code{scale=1} the algorithm satisfies MCMC rules, provided
  that the other properties of the MC (irreducibility and aperiodicity)
  are satisfied.}
\author{Ben Bolker}
\seealso{\code{\link{optim}}, \code{MCMCmetrop1R} (\code{MCMCpack} package)}
\keyword{misc}

\name{contour3d}
\alias{contour3d}
\title{
  Superimpose contour lines on a 3D plot
}
\description{
  Plot contour lines computed from data  in 3D,
  or add them to an existing 3D (RGL) surface
}
\usage{
contour3d(x, y, z, contourArgs=NULL, ...)
}
\arguments{
  \item{x}{numeric vector of x values (as in \code{\link{contour}}),
    or a list with components \code{x}, \code{y} and \code{z}}
  \item{y}{numeric vector of y values (as in \code{\link{contour}})}
  \item{z}{numeric z matrix (as in \code{\link{contour}})}
  \item{contourArgs}{list of arguments to \code{\link{contourLines}}}
  \item{\dots}{other arguments to \code{\link[rgl]{lines3d}}}
}
\value{
  Returns a list of contour lines (as in \code{\link{contourLines}}),
  invisibly.
}
\note{You have to install the \code{rgl} package before you
  can use this function.}
\author{
  Ben Bolker
}
\note{
  If you are superimposing the contour lines on a surface,
  it helps to draw the surface with some level of transparency
  (\code{alpha} parameter: see \code{\link[rgl]{material3d}}) so
  the contour lines are not obscured by the surface.
}
\keyword{misc}

\name{as.mcmc.bugs}
\alias{as.mcmc.bugs}
\title{Convert WinBUGS output to CODA format}
\description{
  Converts results of a \code{bugs} run
  (class \code{"bugs"}) to a form that can be
  used by CODA (class \code{"mcmc"})
}
\usage{
as.mcmc.bugs(x)
}
\arguments{
  \item{x}{an object of class \code{bugs} (output from \code{bugs()})}
}
\value{
  an object of class \code{mcmc}
}
\author{Ben Bolker}
\keyword{misc}

\name{Reedfrog}
\alias{Reedfrog}
\alias{ReedfrogPred}
\alias{ReedfrogFuncresp}
\alias{ReedfrogSizepred}
\docType{data}
\title{Data on reed frog predation experiments}
\description{
  Data on lab experiments on the density- and size-dependent
  predation rate of an African reed frog, \emph{Hyperolius
    spinigularis},
  from Vonesh and Bolker 2005 
}
\usage{data(ReedfrogPred)
data(ReedfrogSizepred)
data(ReedfrogFuncresp)}
\format{
  Various data with variables:
  \describe{
    \item{\code{density}}{initial tadpole density (number of tadpoles
      in a 1.2 x 0.8 x 0.4 m tank) [experiment 1]}
    \item{\code{pred}}{factor: predators present or absent [experiment 1]}
    \item{\code{size}}{factor: big or small tadpoles [experiment 1]}
    \item{\code{surv}}{number surviving}
    \item{\code{propsurv}}{proportion surviving (=surv/density) [experiment 1]}
    \item{\code{TBL}}{tadpole body length in mm [size-predation
	experiment]}
    \item{\code{Kill}}{number killed out of 10, in 3 days
	[size-predation]}
    \item{\code{Initial}}{initial number/density (300 L tank)
	[functional response]}
    \item{\code{Killed}}{number killed by 3 dragonfly larvae in 14
	days [functional response]}
  }
}
\source{
  Vonesh and Bolker (2005) Compensatory larval responses shift
  trade-offs associated with predator-induced hatching plasticity.
  Ecology 86:1580-1591
}
\examples{
data(ReedfrogPred)
boxplot(propsurv~size*density*pred,data=ReedfrogPred)
data(ReedfrogSizepred)
data(ReedfrogFuncresp)
}
\keyword{datasets}

\name{trcoef}
\alias{trcoef}
\title{Transform coefficients}
\description{
  Perform standard transformations of coefficients
  based on information encoded in the names or
  the \code{transf} attribute of the vector or list
}
\usage{
trcoef(x, inverse = FALSE)
}
\arguments{
  \item{x}{A numeric vector of coefficients with names and/or
    a \code{transf} attribute}
  \item{inverse}{(logical) Perform inverse transform?}
}
\details{
  If \code{inverse=FALSE}
  and coefficient names begin with "logit", "log", or "sqrt"
  the function will back-transform them (using \code{\link{plogis}},
  \code{\link{exp}}, or squaring), strip the descriptor
  from the names, and set the \code{transf} attribute.
  Naturally, \code{inverse=TRUE} will do the opposite.
  If the \code{transf} attribute is all empty strings
  after an inverse transformation, it will be deleted.
}
\value{
  A vector of transformed variables with modified names
  and \code{transf} attributes.
}
\author{Ben Bolker}
\examples{
x = c(loga=1,logitb=2,sqrtc=2)
trx = trcoef(x); trx
trcoef(trx,inverse=TRUE)
}
\keyword{misc}

\name{apply2d}
\alias{apply2d}
\title{Apply a function to a combination of vectors}
\description{
  applies a (non-vectorized) function to a combination of vectors;
  substitute for \code{outer}
}
\usage{
apply2d(fun, x, y, ..., use_plyr = TRUE, .progress="none")
}
\arguments{
  \item{fun}{a function of two arguments (or a character string such as \code{"*"})}
  \item{x}{first vector}
  \item{y}{second vector}
  \item{\dots}{additional arguments to \code{fun}}
  \item{use_plyr}{use methods from the \code{plyr} package?}
  \item{.progress}{progress bar type (\code{"none"}, \code{"text"},
    \code{"tk"},\code{"win"}: see \code{\link[plyr]{create_progress_bar}})}

}
\value{
  a matrix of the function applied to the combinations of the vector values
}
\author{Ben Bolker}
\seealso{outer}
\examples{
outer(1:3,1:3)
## this example would work with outer() too
apply2d("*",1:3,1:3)
}
\keyword{misc}

\name{Fir}
\alias{Fir}
\alias{FirDBHFec}
\alias{FirDBHFec_sum}
\docType{data}
\title{Data on fir (Abies) life history}
\description{
  Data on various aspects of life history (diameter at
  breast height, onset of reproduction, crowding, fecundity)
  from subalpine \emph{Abies balsamea},
  from Dodd and Silvertown
}
\usage{data(FirDBHFec)
data(FirDBHFec_sum)
}
\format{
  \describe{
    \item{\code{DBH}}{diameter in m at breast height (1.4 m)}
    \item{\code{fecundity}}{number of cone rachises [per year?]}
    \item{\code{pop}}{which population (wave, nonwave) an
      individual was sampled from}
        \item{\code{VAR1}}{location}
    \item{\code{WAVE_NON}}{non-wave (n) or wave (w)}
    \item{\code{TREE_NO}}{tree number}
    \item{\code{C1991}}{1991 cones}
    \item{\code{C1992}}{1992 cones}
    \item{\code{C1993}}{1993 cones}
    \item{\code{C1994}}{1994 cones}
    \item{\code{C1995}}{1995 cones}
    \item{\code{C1996}}{1996 cones}
    \item{\code{C1997}}{1997 cones}
    \item{\code{C1998}}{1998 cones}
    \item{\code{C1999}}{1999 cones}  
    \item{\code{NOTES_IN}}{notes}
    \item{\code{G1990}}{1990 growth}
    \item{\code{G1991}}{1991 growth}
    \item{\code{G1992}}{1992 growth}
    \item{\code{G1993}}{1993 growth}
    \item{\code{G1994}}{1994 growth}
    \item{\code{G1995}}{1995 growth}
    \item{\code{G1996}}{1996 growth}
    \item{\code{G1997}}{1997 growth}
    \item{\code{G1998}}{1998 growth}
    \item{\code{DBH}}{diameter at breast height}
    \item{\code{DBH_MM}}{dbh in mm}
    \item{\code{DBH_2}}{?}
    \item{\code{DBH_2MM}}{?}
    \item{\code{AGE}}{?}
    \item{\code{GOOD_OR}}{?}
    \item{\code{PC1998}}{?}
    \item{\code{AC1998}}{?}
    \item{\code{PC1994}}{?}
    \item{\code{AC1994}}{?}
    \item{\code{R3PC1998}}{?}
    \item{\code{RPC1994}}{?}
    \item{\code{RAC1994}}{?}
    \item{\code{RLOOKOUT}}{?}
    \item{\code{RSHREWYS}}{?}
    \item{\code{RWILLS}}{a factor with levels \code{0} \code{1} \code{Fajita}}
    \item{\code{C8TOT}}{?}
    \item{\code{G8TOT}}{?}
    \item{\code{RAC1994I}}{?}
    \item{\code{RPC1994I}}{?}
    \item{\code{R3PC1998.1}}{?}
    \item{\code{AC1998I}}{?}
    \item{\code{TOTCONES}}{total cones}
  }    
}
\source{
  J. Silvertown and M. Dodd, Evolution of life history
  in balsam fir (\emph{Abies balsamea}) in subalpine
  forests, Proc. Roy. Soc. Lond. B (1999) 266, 729-733.
}
\references{
  M. Dodd and J. Silvertown, Size-specific fecundity
  and the influence of lifetime size variation upon
  effective population size in \emph{Abies balsamea}
}
\examples{
data(FirDBHFec_sum)
attach(FirDBHFec_sum)
plot(DBH,fecundity,col=as.numeric(pop),pch=as.numeric(pop))
lms = lapply(split(FirDBHFec_sum,pop),lm,formula=fecundity~DBH)
for (i in 1:2) abline(lms[[i]],col=i)
detach(FirDBHFec_sum)
}
\keyword{datasets}

\name{get.emdbook.packages}
\alias{get.emdbook.packages}
\alias{update.bmb.packages}
\title{install and update auxiliary packages}
\description{
   convenience function for downloading and installing
   all the packages needed for the book (just a list
   and a wrapper around \code{\link{install.packages}})
}
\usage{
get.emdbook.packages()
}
\value{
  none: installs packages as a side effect
}
\author{Ben Bolker}
\seealso{\code{\link{install.packages}}}
\keyword{misc}

\name{scinot}
\alias{scinot}
\alias{axis.scinot}
\title{Scientific notation as LaTeX/expression()}
\description{
  Takes a number and returns a version formatted in LaTeX
  (suitable for use with \code{Sexpr()} in an Sweave document)
  or in \code{expression()} (suitable for plotting),
  or plots an axis with labels in scientific notation
}
\usage{
scinot(x, format = c("latex", "expression"), delim="$",
pref="", ...)
axis.scinot(side,at)
}
\arguments{
  \item{x}{a numeric vector (of length 1)}
  \item{format}{produce LaTeX or expression() format?}
  \item{delim}{delimiter to add at beginning and end (latex only)}
  \item{pref}{text to put before expression (expression only)}
  \item{side}{side on which to plot axis}
  \item{at}{list of locations/labels}
  \item{\dots}{additional arguments to \code{\link{formatC}}}
}
\value{
  a character vector (if \code{latex}) or expression (if
  \code{expression}); \code{axis.scinot} draws an axis on the
  current plot
  
}
\author{Ben Bolker}
\seealso{\code{\link{formatC}}, \code{\link{expression}},
  \code{\link{plotmath}}, \code{\link{axis}}, \code{\link{axTicks}},
  \code{latexSN} in the \code{Hmisc} package, \code{eaxis} in the \code{sfsmisc} package}
\examples{
scinot(1e-5)
scinot(1e-5,digits=0)
scinot(1e-5,"expression")
scinot(1e-5,"expression",pref="p=")
set.seed(1001)
plot(1:100,rlnorm(100,0,2),log="y",axes=FALSE)
axis(side=1)
axis.scinot(side=2)  ## fix bug!
}
\keyword{misc}

\name{HPDregionplot}
\alias{HPDregionplot}
\title{Plot highest posterior density region}
\description{
  Given a sample from a posterior distribution (an \code{mcmc} object
  from the \code{coda} package),
  plot the bivariate region of highest marginal posterior density
  for two variables, using \code{kde2d} from \code{MASS} to calculate
  a bivariate density.
}
\usage{
HPDregionplot(x, vars = 1:2, h, n = 50, lump = TRUE, prob = 0.95, xlab =
NULL, ylab = NULL, lims=NULL, ...)
}
\arguments{
  \item{x}{an \code{mcmc} or \code{mcmc.list} object}
  \item{vars}{which variables to plot: numeric or character vector}
  \item{h}{bandwidth of 2D kernel smoother (previous default value was \code{c(1,1)},
    which worked poorly with some plots with very small scales;
  if not specified, defaults to values in \code{\link[MASS]{kde2d}})}
  \item{n}{number of points at which to evaluate the density grid}
  \item{lump}{if \code{x} is an \code{mcmc.list} object, lump the
    chains together for plotting?}
  \item{prob}{probability level}
  \item{xlab}{x axis label}
  \item{ylab}{y axis label}
  \item{lims}{limits, specified as (x.lower,x.upper,y.lower,y.upper)
    (passed to \code{kde2d})}
  \item{\dots}{other arguments to \code{\link{contour}}}
}
\details{
  Uses \code{kde2d} to calculate a bivariate density, then
  normalizes the plot and calculates the contour corresponding
  to a contained volume of \code{prob} of the total volume under
  the surface (a two-dimensional Bayesian credible region).
}
\value{
  Draws a plot on the current device, and
  invisibly returns a list of contour lines (\code{\link{contourLines}}).
}
\author{Ben Bolker}
\note{Accuracy may be limited by density estimation; you may
  need to tinker with \code{h} and \code{n} (see \code{kde2d}
  in the \code{MASS} package).
  }
  \seealso{\code{HPDinterval} in the \code{coda} package,
  \code{ellipse} package}
\examples{
library(MASS)
library(coda)
z <- mvrnorm(1000,mu=c(0,0),Sigma=matrix(c(2,1,1,2),nrow=2))
z2 <- mvrnorm(1000,mu=c(0,0),Sigma=matrix(c(2,1,1,2),nrow=2))
HPDregionplot(mcmc(z))
HPDregionplot(mcmc.list(mcmc(z),mcmc(z2)))
}
\keyword{hplot}

\name{dzinbinom}
\alias{dzinbinom}
\alias{rzinbinom}
\title{Zero-inflated negative binomial distribution}
\description{
  Probability distribution function and random variate generation for the
  zero-inflated negative binomial distribution
}
\usage{
dzinbinom(x, mu, size, zprob, log=FALSE)
rzinbinom(n, mu, size, zprob)
}
\arguments{
  \item{x}{vector of integer values}
  \item{n}{number of values to draw}
  \item{mu}{mean parameter (or vector of parameters) of negative binomial}
  \item{size}{number of trials/overdispersion parameter (or vector of
parameters) of negative binomial}
  \item{zprob}{probability of structural zeros}
  \item{log}{return log probability?}
}
\details{
The zero-inflated negative binomial distribution is
widely used to model extra zero counts in count data
that otherwise follows a negative binomial distribution.
The probability distribution is
 \deqn{
    p(0) =p_z + (1-p_z) NB(0,mu,k)}{% 
    p(0) = pz + (1-pz) NB(0,mu,k)}
and
\deqn{
    p(x) =(1-p_z) NB(x,mu,k)}{% 
    p(x) = (1-pz) NB(x,mu,k)}
for \eqn{x>0}.
}
\value{
  Probabilities of x or random deviates.
}
\note{Only the "ecological" parameterization is included here
(must specify \code{mu}, not \code{prob})}
\references{Tyre et al., "Improving precision and reducing bias in biological
surveys: estimating false-negative error rates", Ecological Applications
13:1790-1801 (2003)}
\author{Ben Bolker}
\seealso{\code{\link{dnbinom}}, Simon Jackman's pscl package}
\examples{
dzinbinom(0:9,mu=2,zprob=0.3,size=0.9)
dnbinom(0:9,mu=2,size=0.9)
rzinbinom(10,mu=2,zprob=0.3,size=0.9)
}
\keyword{misc}

\name{dchibarsq}
\alias{dchibarsq}
\alias{pchibarsq}
\alias{qchibarsq}
\alias{rchibarsq}
\title{Mixed chi-squared distributions}
\description{
  Calculates "mixed" chi-squared distributions
  (mixtures of chi-square(n) and chi-square(n-1));
  useful for Likelihood Ratio Tests when
  parameters are on the boundary
}
\usage{
dchibarsq(x, df = 1, mix = 0.5, log = FALSE)
pchibarsq(p, df = 1, mix = 0.5, lower.tail=TRUE, log.p = FALSE)
qchibarsq(q, df = 1, mix = 0.5)
rchibarsq(n, df = 1, mix = 0.5)
}
\arguments{
  \item{x}{numeric vector of positive values}
  \item{p}{numeric vector of positive values}
  \item{q}{numeric vector of quantiles (0-1)}
  \item{n}{integer: number of random deviates to pick}
  \item{df}{degrees of freedom (positive integer)}
  \item{mix}{mixture parameter: fraction of distribution
    that is chi-square(n-1) distributed}
  \item{log}{return log densities?}
  \item{log.p}{return log probabilities?}
  \item{lower.tail}{return lower tail values?}
}
\value{
  Vectors of probability densities (\code{dchibarsq}),
  cumulative probabilities (\code{pchibarsq}),
  quantiles (\code{qchibarsq}), or
  random deviates (\code{rchibarsq}) from
  Goldman and Whelan's "chi-bar-squared" distribution.
  \code{qchibarsq} uses simple algebra for df=1
  and \code{\link{uniroot}} for df>1.
}
\references{N. Goldman and S. Whelan (2000) "Statistical Tests of
  Gamma-Distributed Rate Heterogeneity in Models of Sequence Evolution
  in Phylogenetics", Mol. Biol. Evol. 17:975-978.
  D. O. Stram and J. W. Lee (1994) "Variance Components Testing in the
  Longitudinal Fixed Effects Model", Biometrics 50:1171-1177.
  }
  
  \author{Ben Bolker}
\examples{
x <- rchibarsq(100)
plot(density(x,from=0))
curve(dchibarsq(x),add=TRUE,col=2,from=0)
\dontrun{
library(lattice)
print(qqmath(~ simdist,
       distribution=qchibarsq,
       panel = function(x, ...) {
         panel.qqmathline(x, ...)
         panel.qqmath(x, ...)
            }))

}
## create first line of table in Goldman and Whelan 2000
round(qchibarsq(c(0.01,0.05,0.9,0.95,0.975,0.99,0.995),df=1),2)
## check second line of table
round(pchibarsq(c(3.81,5.14,6.48,8.27,9.63),df=2),3)
## create middle column
round(qchibarsq(0.95,df=1:10))
}
\keyword{misc}

\name{Damselfish}
\alias{Damselfish}
\alias{DamselSettlement}
\alias{DamselRecruitment}
\alias{DamselRecruitment_sum}
\docType{data}
\title{Reef fish (damselfish) data}
\description{
  Two data sets on \emph{Dascyllus trimaculatus}
(three-spot damselfish), one on the distribution of settlement densities
  to empty anemones across time and space, the other on survival
(recruitment) of arriving settlers as a function of experimentally manipulated densities
  from Schmitt et al. (1999).
}
\usage{data(DamselSettlement)
data(DamselRecruitment)
data(DamselRecruitment_sum)
}
\format{
  Three data frames:
  \describe{
    \item{\code{site}}{settlement site (location)}
    \item{\code{pulse}}{monthly settlement pulse}
    \item{\code{obs}}{observation within pulse}
    \item{\code{density}}{density of settlers per 0.1 m2 anemone}
    \item{\code{area}}{anemone area in cm2}
    \item{\code{init}}{initial settler density}
    \item{\code{surv}}{surviving density after 6 months}
    \item{\code{settler.den}}{target experimental density of settlers on
      experimental anemones}
    \item{\code{surv.den}}{mean surviving density after 6 months, by target density}
    \item{\code{SE}}{standard error of survivor density, by target density}
}
}
\source{
  Schmitt et al. (1999), "Quantifying the effects of multiple processes
  on local abundance", Ecology Letters 2:294-303. DOI: 10.1046/j.1461-0248.1999.00086.x (Original data kindly provided
  by Schmitt and Holbrook.)
}
\keyword{datasets}

\name{lambertW}
\alias{lambertW}
\alias{lWasymp}
\alias{lambertW_base}
\title{Lambert W function}
\description{
  Computes the Lambert W function, giving efficient solutions to the equation x*exp(x)==z
}
\usage{
lambertW_base(z, b = 0, maxiter = 10, eps = .Machine$double.eps, min.imag =
1e-09)
lWasymp(z,logz)
lambertW(z,\dots)
}
\arguments{
  \item{z}{(complex) vector of values for which to compute the function}
  \item{logz}{(complex (?)) vector of \eqn{log(z)} values (to be
    specified by name instead of \code{z})}
  \item{b}{(integer) b=0 specifies the principal
    branch, 0 and -1 are the ones that can take non-complex values}
  \item{maxiter}{maximum numbers of iterations for convergence}
  \item{eps}{convergence tolerance}
  \item{min.imag}{maximum magnitude of imaginary part to chop when
    returning solutions}
  \item{\dots}{arguments to pass to \code{lambertW_base}}
}
\details{
Compute the Lambert W function of z.  This function satisfies
\eqn{W(z)\exp(W(z))=z}{W(z)*exp(W(z)) = z}, and can thus be used to express solutions
of transcendental equations involving exponentials or logarithms.
For \eqn{z>10^307}{z>1e307}, an asymptotic formula (from
Corless et al by way of
\url{http://mathworld.wolfram.com/LambertW-Function.html})
is used: \code{lambertW} is a wrapper that automatically selects
the asymptotic formula where appropriate.
\itemize{
  \item{In ecology, the Lambert W can be used to solve the so-called
    "Rogers equation" for predator functional response with depletion.}
  \item{In epidemiology, the Lambert W function solves the final-size
    equation of a simple SIR epidemic model.}
  }
}
\value{
  Complex or real vector of solutions.
}
\references{Corless, Gonnet, Hare, Jeffrey, and Knuth (1996), "On the Lambert
W Function", Advances in Computational Mathematics 5(4):329-359}
\author{Nici Schraudolph <schraudo@inf.ethz.ch> (original
  version (c) 1998), Ben Bolker (R translation)
}
\seealso{\code{?Lambert} in the \code{gsl} package by Robin Hankin,
  which uses Gnu Scientific Library code; also \code{?lambertW}
in the \code{VGAM} and \code{pracma} packages, and the \code{lambertW} package}
\note{
This implementation should return values within 2.5*eps of its
counterpart in Maple V, release 3 or later.  Please report any
discrepancies to the author or translator.

The derivative of the \code{lambertW} function is \code{plogis(-lambertW)}.
}
\examples{
curve(lambertW(x),from=0,to=10)
pvec <- seq(-1,1,length=40)
m <- outer(pvec,pvec,function(x,y)Re(lambertW(x+y*1i)))
persp(pvec,pvec,m,
      theta=290,shade=0.5,zlab="lambertW")
num1 <- uniroot(function(x) {x*exp(x)-1},lower=0,upper=1,tol=1e-9)
abs(lambertW(1)-num1$root)<1e-9
###
## Rogers random predator equation:
rogers.pred <- function(N0,a,h,T) {
   N0 - lambertW(a*h*N0*exp(-a*(T-h*N0)))/(a*h)
}
holling2.pred <- function(N0,a,h) {
  a*N0/(1+a*h*N0)
}
curve(rogers.pred(x,a=1,h=0.2,T=1),from=0,to=60,
  ylab="Number eaten/unit time",xlab="Initial number",ylim=c(0,5),
  main="Predation: a=1, h=0.2")
curve(rogers.pred(x,a=1,h=0.2,T=5)/5,add=TRUE,lty=2,from=0)
curve(rogers.pred(x,a=1,h=0.2,T=0.2)*5,add=TRUE,lty=3,from=0)
curve(rogers.pred(x,a=1,h=0.2,T=10)/10,add=TRUE,lty=4,from=0)
curve(holling2.pred(x,a=1,h=0.2),add=TRUE,lty=1,lwd=2,from=0)
abline(h=5)
legend(30,2,
   c(paste("Rogers, T=",c(0.2,1,5,10),sep=""),
    "Holling type II"),lwd=c(rep(1,4),2),lty=c(3,1,2,4,1))
## final size of an epidemic
finalsize <- function(R0) {
   1+1/R0*lambertW(-R0*exp(-R0))
}
curve(finalsize,from=1,to=10,xlab=expression(R[0]),ylab="Final size")
## comparison of asymptotic results
tmpf <- function(x) {
  L0 <- lambertW_base(10^x)
  L1 <- lWasymp(logz=x*log(10))
  (L1-L0)/L0
}
curve(tmpf,from=1,to=307,log="y")

## derivative
## don't run (avoid numDeriv dependency)
## require(numDeriv)
##   grad(lambertW(1))
##   plogis(-lambertW(1))
}
\keyword{math}

\name{Lily}
\alias{Lily}
\alias{Lily_sum}
\docType{data}
\title{Glacier lily occurrence and fecundity data}
\description{
	Data on sample quadrats of the glacier lily,
	\emph{Erythronium grandiflorum}, from Thomson et al 1996
}
\usage{data(Lily_sum)}
\format{
  \describe{
    \item{\code{x}}{location of quadrat}
    \item{\code{y}}{location of quadrat}
    \item{\code{flowers}}{number of flowers}
    \item{\code{seedlings}}{number of seedlings}
    \item{\code{vegetative}}{number of vegetative plants}
    \item{\code{gopher}}{index of gopher activity}
    \item{\code{rockiness}}{rockiness index}
    \item{\code{moisture}}{moisture index}
    \item{\code{flowcol}}{inverse quintile of flowering plants}
    \item{\code{seedcol}}{inverse quintile of seedlings}
    \item{\code{vegcol}}{inverse quintile of number of vegetative plants for image plots}
    \item{\code{gophcol}}{inverse quintile of gopher activity}
    \item{\code{rockcol}}{inverse quintile of rockiness}
    \item{\code{moiscol}}{inverse quintile of moisture}
  }
}
\details{
  16x16 grid of 2x2m quadrats in Washington Gulch, sampled 1992
}
\source{
  Thomson et al 1996, "Untangling multiple factors in spatial distributions",
  Ecology 77:1698-1715.  Data from James D. Thomson, with file format
  conversion help from Jennifer Schmidt
}
\examples{
data(Lily_sum)
par(mfrow=c(3,2))
for (i in 9:14) {
  image(matrix(Lily_sum[,i],nrow=16),main=names(Lily_sum)[i])
}
}
\keyword{datasets}

\name{dmvnorm}
\alias{dmvnorm}
\title{Multivariate normal distribution density function}
\description{
  Calculates the probability density function of the multivariate normal distribution
}
\usage{
dmvnorm(x, mu, Sigma, log = FALSE, tol = 1e-06)
}
\arguments{
  \item{x}{a vector or matrix of multivariate observations}
  \item{mu}{a vector or matrix of mean values}
  \item{Sigma}{a square variance-covariance matrix}
  \item{log}{(logical) return log-likelihood?}
  \item{tol}{tolerance for positive definiteness}
}
\details{
  uses naive linear algebra -- could probably use QR decomposition
  and/or crossprod.
}
\value{
  vector of log-likelihoods
}
\author{Ben Bolker}
\seealso{\code{\link[MASS]{mvrnorm}} (in \code{MASS} package),
  \code{dmvnorm} (in \code{mvtnorm} package)}
\examples{
M = matrix(c(1,0.5,0.5,0.5,1,0.5,0.5,0.5,1),nrow=3)
dmvnorm(1:3,mu=1:3,Sigma=M,log=TRUE)
dmvnorm(matrix(1:6,nrow=2),mu=1:3,Sigma=M,log=TRUE)
dmvnorm(matrix(1:6,nrow=2),mu=matrix(1:6,nrow=2),Sigma=M,log=TRUE)
}
\keyword{misc}

\name{lump.mcmc.list}
% \alias{traceplot}
% \alias{traceplot.mcmc}
% \alias{traceplot.mcmc.list}
\alias{lump.mcmc.list}
\title{Utility functions for mcmc objects}
\description{
  Creates traceplots or combine mcmc list into
  mcmc objects
}
\usage{
lump.mcmc.list(x)
%\method{traceplot}{mcmc}(x,smooth=TRUE,ylab="",sys="lattice",\dots)
%\method{traceplot}{mcmc.list}(x,smooth=TRUE,ylab="",sys="lattice",\dots)
}
\arguments{
  \item{x}{an \code{mcmc.list} object}
%  \item{smooth}{(logical) add smoothed lines to trace plots?}
%  \item{ylab}{(character) y-label for trace plots}
%  \item{sys}{(character) graphics system to use}
%  \item{\dots}{extra arguments (for generic consistency)}
}
\value{
%  \code{traceplot.mcmc} and  \code{traceplot.mcmc.list}
%  create plots; \code{lump.mcmc.list} returns
a single \code{mcmc} object with the chains lumped together
}
\author{Ben Bolker}
\seealso{\code{coda} package}
\keyword{misc}

\name{deltamethod}
\alias{deltamethod}
\alias{deltavar}
\title{Delta method functions}
\description{
  Delta-method implementations for Jensen's inequality and prediction uncertainty
}
\usage{
deltamethod(fun, z, var = "x", params = NULL, max.order = 2)
deltavar(fun,meanval=NULL,vars,Sigma,verbose=FALSE)
}
\arguments{
  \item{fun}{Function of one (deltamethod) or more arguments, expressed
    in raw form (e.g. a*x/(b+x))}
  \item{z}{numeric vector of values}
  \item{var}{variable name}
  \item{vars}{list of variable names: needed if \code{params} does not
    have names, or if some of the values specified in \code{params}
    should be treated as constant}
  \item{params}{list or numeric vector of parameter values to substitute}
  \item{meanval}{possibly named vector of mean values of parameters}
  \item{Sigma}{numeric vector of variances or variance-covariance matrix}
  \item{max.order}{maximum order of delta method to compute}
  \item{verbose}{print details?}
}
\details{
  \code{deltamethod()} is for computing delta-method approximations of
  the mean of a function of data; \code{deltavar()} is for estimating
  variances of a function based on the mean values and
  variance-covariance matrix of the parameters.  If \code{Sigma} is a
  vector rather than a matrix, the parameters are assumed to be
  independently estimated.
}
\value{
  For \code{deltavar()}, a vector of predicted variances; for
  \code{deltamethod()} a vector containing the observed value of the
  function average, the function applied to the average, and a series of
  delta-method approximations
}
\references{Lyons (1991), "A practical guide to data analysis for
  physical science students", Cambridge University Press}
\author{Ben Bolker}
\examples{
deltamethod(a*x/(b+x),runif(50),params=list(a=1,b=1),max.order=9)
deltavar(scale*gamma(1+1/shape),meanval=c(scale=0.8,shape=12),
   Sigma=matrix(c(0.015,0.125,0.125,8.97),nrow=2))
## more complex deltavar example
xvec = seq(-4,4,length=101)
x1 = xvec
x2 = xvec
v = matrix(0.2,nrow=3,ncol=3)
diag(v) = 1
m = c(b0=1,b1=1.5,b2=1)
v3  = deltavar(1/(1+exp(-(b0+b1*x1+b2*x2))),meanval=m,Sigma=v)
plot(xvec,v3)
}
\keyword{misc}

\name{gridsearch2d}
\alias{gridsearch2d}
\title{Graphical grid search in 2D}
\description{
  Given an objective function and starting ranges, computes the
  values over the ranges and displays them in the graphics window.
  User can then interactively zoom in to view interesting
  parts of the surface.
}
\usage{
gridsearch2d(fun, v1min, v2min, v1max, v2max,
n1 = 20, n2 = 20, logz = FALSE,
sys3d = c("both", "contour", "image"), ...)
}
\arguments{
  \item{fun}{Objective function to be minimized: function of two arguments}
  \item{v1min}{Minimum starting value of variable 1}
  \item{v2min}{Minimum starting  value of variable 2}
  \item{v1max}{Maximum starting  value of variable 1}
  \item{v2max}{Maximum starting  value of variable 2}
  \item{n1}{Number of grid points for variable 1}
  \item{n2}{Number of grid points for variable 2}
  \item{logz}{Display image or contour on log scale?}
  \item{sys3d}{Display surface as an image, contour, or both?}
  \item{\dots}{Other arguments to \code{fun}}
}
\details{
  If \code{log=TRUE}, the value of the surface is rescaled to
  \code{log10(m-min(m)+mindm)}, where \code{mindm} is the
  difference between the minimum and the next-largest value
  (or 1e-10 if this difference is zero).

  At each iteration, the user is prompted to select two
  corners of the new range with the mouse; if this choice
  is confirmed then the view zooms in.  When the user chooses
  to quit, they are asked whether they want to choose a final
  point (e.g. an estimate of the minimum) with the mouse.
}
\value{
  If a final point is chosen, a list with elements \code{x} and
  \code{y},
  otherwise NULL.
}
\author{Ben Bolker}
\seealso{\code{\link{curve3d}}}
\keyword{iplot}

\name{credint}
\alias{tcredint}
\alias{ncredint}
\title{Calculate Bayesian credible intervals}
\description{
  Calculate Bayesian credible intervals based on various types
  of information about the posterior distribution
}
\usage{
tcredint(dist, parlist, ranges, level = 0.95, eps = 1e-05,verbose=FALSE)
ncredint(pvec,npost,level=0.95,tol=0.01,verbose=FALSE)
}
\arguments{
  \item{dist}{character string giving the name of a distribution for
    which "d", "q", and "p" function exist, e.g. "beta"}
  \item{parlist}{list of parameters to pass to distribution functions}
  \item{ranges}{lower, middle, and upper values to bracket lower
  and upper boundaries of the credible interval}
  \item{level}{confidence level}
  \item{eps}{if \code{ranges} is missing, set lower and
    upper brackets to the \code{eps} and \code{1-eps} quantiles
    of the distribution}
  \item{tol}{tolerance on credible interval}
  \item{verbose}{if TRUE,
    return detailed information on the probability cutoff and
    realized area of the credible interval; if FALSE, just lower
    and upper bounds of the credible region}
  \item{pvec}{numeric vector of parameter values}
  \item{npost}{numeric vector of posterior density values corresponding
    to \code{pvec}}
}
\details{
  \code{tcredint} gives credible intervals for a theoretical
  posterior density with defined density, cumulative density, and
  quantile functions; \code{ncredint} gives credible intervals
  for a numerical posterior density.
}
\value{
  A numeric vector giving the credible interval.
  If \code{verbose=FALSE}, gives just lower and upper bounds;
  if \code{verbose=TRUE}, also gives
  information on the probability cutoff and
    realized area of the credible interval
  }
  \note{For credible intervals from a sample (e.g. from
    an MCMC run), see \code{HPDinterval} in the \code{coda}
    package.}
\author{Ben Bolker}
\examples{
tcredint("beta",list(shape1=5,shape2=10),verbose=TRUE)
pvec = seq(0,1,length=100)
postvec = dbeta(pvec,shape1=5,shape2=10)
ncredint(pvec,postvec,verbose=TRUE)
set.seed(1001)
}
\keyword{misc}

\name{perturb.params}
\alias{perturb.params}
\title{Create a list of perturbed parameters}
\description{
  Takes a baseline set of parameters and perturbs it to
  create a variety of starting points for maximum likelihood
  estimation or MCMC
}
\usage{
perturb.params(base, alt, which, mult = FALSE, use.base = TRUE)
}
\arguments{
  \item{base}{a named list (or vector) of parameters}
  \item{alt}{a list of lists (or vectors) of alternative parameter values
    or multipliers}
  \item{which}{which parameters to perturb (currently unused)}
  \item{mult}{(logical) multiply baseline values rather than
    replacing them?}
  \item{use.base}{(logical) include baseline parameters in the list?}
}
\details{
  Takes the baseline parameter list and substitutes alternative values.
}
\value{
  A list of named lists of parameters.
}
\author{Ben Bolker}
\note{To be extended.}
\examples{
perturb.params(list(x=1,y=2,z=3),alt=list(x=c(2,4),z=5))
}
\keyword{optimize}

\name{emdbook-package}
\alias{emdbook-package}
\alias{emdbook}
\docType{package}
\title{
  \packageTitle{emdbook}
}
\description{
  \packageDescription{emdbook}
}
\references{
  \cite{Bolker, Benjamin M. Ecological Models and Data in R. Princeton University Press, 2008}
}
\keyword{ package }

\name{GobySurvival}
\alias{GobySurvival}
\docType{data}
\title{Goby (reef fish) survivorship data}
\description{
  Survivorship data from experimental
  manipulations on gobies \emph{Elacatinus evelynae} and
  \emph{E. prochilos} in the US Virgin Islands, 2000-2002
}
\format{
  \describe{
    \item{\code{exper}}{experiment}
    \item{\code{year}}{year}
    \item{\code{site}}{site (factor: backreef, patchreef)}
    \item{\code{head}}{coral head (factor)}
    \item{\code{density}}{treatment "density" (number of "target" fish)}
    \item{\code{qual}}{treatment "quality"; background settlement rate}
    \item{\code{d1}}{last day observed (starting at 1)}
    \item{\code{d2}}{first day not observed}
  }
}
\source{
  J. Wilson, pers. comm.; "Habitat quality, competition and recruitment
  processes in two marine gobies", Ph.D. thesis, University of Florida
  (2004);
  \url{https://ufdc.ufl.edu/UFE0004180/00001/pdf}
}
\details{
  These data have been made available by the author for pedagogical use;
  out of courtesy,
  please don't redistribute (outside of the context of this
  package) or use in an academic publication without requesting
  permission (via the package maintainer).
}
\examples{
## midpoint of survival times
gg <- transform(GobySurvival,mid=(d1+d2)/2)
plot(table(gg$mid))
}
\keyword{datasets}

\name{Myxo}
\alias{Myxo}
\alias{MyxoTiter_sum}
\docType{data}
\title{Myxomatosis titer data}
\description{
   Myxomatosis viral titer in blood samples from European rabbits,
   as a function of day-of-infection and virus grade,
   from Dwyer et al. 1990, ultimately from Fenner et al. 1956
}
\usage{data(MyxoTiter_sum)}
\format{
  \describe{
    \item{\code{grade}}{virus grade (1, least virulent; 5, most virulent)}
    \item{\code{day}}{day of infection}
    \item{\code{titer}}{blood virus titer (in log10 rabbit infectious doses)}
  }
}
\source{
  Dwyer, Levin and Buttel, "A Simulation Model of the Population
  Dynamics and Evolution of Myxomatosis", Ecological Monographs
  60(4):423-447 (1990).
  Original source: Fenner et al. 1956
}
\note{Pulled graphically from figure in Dwyer et al.;
  to be replaced (eventually) with original tabular data in Fenner et al.
}
\examples{
data(MyxoTiter_sum)
library(lattice)
xyplot(titer~day|factor(grade),data=MyxoTiter_sum,xlim=c(0,30))
}
\keyword{datasets}

\name{curve3d}
\alias{curve3d}
\title{Plot a 3D surface representing a 2D curve}
\description{
  Two-dimensional analogue of \code{curve}: generates a surface
  and plots it
}
\usage{
curve3d(expr, from = c(0, 0), to = c(1, 1), n = c(41, 41),
xlim, ylim, add = FALSE,
xlab=varnames[1],
ylab=varnames[2],
zlab = NULL, log = NULL, sys3d = c("persp", "wireframe", "rgl",
"contour", "image", "none"),
varnames=c("x","y"),use_plyr=TRUE,.progress="none",
data = NULL, ...)
}
\arguments{
  \item{expr}{a mathematical expression using \code{x} and \code{y} as
    the independent variables}
  \item{from}{minimum values for \code{x} and \code{y}}
  \item{to}{maximum values for \code{x} and \code{y}}
  \item{xlim}{range of values for \code{x}}
  \item{ylim}{range of values for \code{y}}
  \item{n}{number of grid points in each direction}
  \item{add}{(logical) add to an existing plot?  (only possible for
    contour plots or rgl)}
  \item{xlab}{x label}
  \item{ylab}{y label}
  \item{zlab}{z label}
  \item{log}{(character): \code{"x"}, \code{"y"}, or \code{"xy"} for
    logarithmic axes}
  \item{sys3d}{3D plotting system to use: one of
    \code{"persp"}, \code{"wireframe"},
    \code{"rgl"}, \code{"contour"}, \code{"image"},
    or \code{"none"}}
  \item{varnames}{names of variables to substitute}
  \item{use_plyr}{use methods from the \code{plyr} package?}
  \item{.progress}{progress bar type (\code{"none"}, \code{"text"},
    \code{"tk"},\code{"win"}: see \code{\link[plyr]{create_progress_bar}})}
  \item{data}{additional data (as a list) to use in evaluating \code{expr}}
  \item{\dots}{additional arguments to the plotting functions}
}
\value{
  invisibly, a list of
  \item{x}{x values}
  \item{y}{y values}
  \item{z}{z matrix}
}
\note{
  \itemize{
    \item You must explicitly install the \code{rgl} package
    (via \code{install.packages("rgl")}) before using
    \code{sys3d="persp"}.
    \item if you encounter the error \sQuote{Results must have one or
      more dimensions}, try \code{use_plyr=FALSE} or use \code{c()}
    to remove attributes from the result of your expression
  }
}
\seealso{\code{\link{outer}}, \code{\link{curve}}}
\examples{
curve3d(cos(2*pi*x)+sin(2*pi*y/3),
 from=c(0,0),to=c(1,1))
x <- 1
y <- 3
curve3d(cos(2*pi*x)+sin(2*pi*y/3),
 from=c(0,0),to=c(1,1),sys3d="wireframe")
curve3d(x*cos(2*pi*a/x)+sin(2*pi*b/y),
 from=c(0,0),to=c(1,1),sys3d="wireframe",
varnames=c("a","b"))  ## identical
op <- par(mfrow=c(2,2))
curve3d(cos(2*pi*x)+sin(2*pi*y/3),
 from=c(0,0),to=c(1,1),sys3d="image")
curve3d(x*cos(2*pi*a/x)+sin(2*pi*b/y),
 from=c(0,0),to=c(1,1),sys3d="image",
varnames=c("a","b"))  ## identical
x <- 4
curve3d(cos(2*pi*a/x)+y*sin(2*pi*b/y),
 from=c(0,0),to=c(1,1),sys3d="image",
varnames=c("a","b"))
curve3d(cos(2*pi*x)+sin(2*pi*y/3),
 from=c(0,0),to=c(1,1),sys3d="image")
curve3d(cos(2*pi*x)+sin(2*pi*y/3),
        sys3d="contour",add=TRUE)
par(op)
}
\keyword{hplot}

\name{lseq}
\alias{lseq}
\title{Log-spaced sequence}
\description{
  Generates a logarithmically spaced sequence
}
\usage{
lseq(from, to, length.out)
}
\arguments{
  \item{from}{starting value}
  \item{to}{ending value}
  \item{length.out}{number of intervening values}
}
\details{
  \code{lseq()} is just a wrapper for
\code{exp(seq(log(from), log(to), length.out = length.out))}
}
\value{
  logarithmically spaced sequence
}
\author{Ben Bolker}
\seealso{\code{\link{seq}}}
\examples{
lseq(10,1000,9)
}
\keyword{misc}
